var inquirer = require("inquirer");
var fs = require("fs");
var pm2 = require("pm2");
var exec = require("child_process").exec;
var targz = require("targz");
var os = require("os");
var envVariable = require("./settings");

var config = {
  ivm: {
    name: "ivm",
    log_date_format: "YYYY-MM-DD HH:mm:ss.SSS",
    env: {
      PORT: 27210,
      ROOT_URL: "http://localhost",
      LOG_LEVEL: "error",
      NODE_ENV: "production"
    }
  },
  hca: {
    name: "hca",
    log_date_format: "YYYY-MM-DD HH:mm:ss.SSS",
    env: {
      PORT: 27211,
      ROOT_URL: "http://localhost",
      LOG_LEVEL: "error",
      NODE_ENV: "production"
    }
  },
  eai: {
    name: "eai",
    log_date_format: "YYYY-MM-DD HH:mm:ss.SSS",
    env: {
      PORT: 27213,
      ROOT_URL: "http://localhost",
      LOG_LEVEL: "error",
      NODE_ENV: "production"
    }
  },
  rakeManagement: {
    name: "rakeManagement",
    log_date_format: "YYYY-MM-DD HH:mm:ss.SSS",
    env: {
      PORT: 27214,
      ROOT_URL: "http://localhost",
      LOG_LEVEL: "error"
    }
  },
  hcs: {
    name: "hcs",
    log_date_format: "YYYY-MM-DD HH:mm:ss.SSS",
    env: {
      NODE_ENV: "production"
    }
  },
  roadmon: {
    name: "roadmon",
    log_data_format: "YYYY-MM-DD HH:mm:ss.SSS",
    env: {
      NODE_ENV: "production"
    }
  }
};
var questions = [
  {
    type: "checkbox",
    message: "Select projects to install",
    name: "projects",
    choices: [
      {
        name: "ivm",
        checked: true
      },
      {
        name: "hca"
      },
      {
        name: "hcs"
      },
      {
        name: "eai"
      },
      {
        name: "roadmon"
      },
      {
        name: "rakeManagement"
      }
    ]
  },
  {
    when: function (response) {
      return response.projects.indexOf("eai") > -1;
    },
    type: "input",
    name: "eaiMongoUrl",
    message: "Specify mongodb url for eai app"
  },
  {
    when: function (response) {
      return response.projects.indexOf("ivm") > -1;
    },
    type: "input",
    name: "ivmMongoUrl",
    message: "Specify mongodb url for ivm app"
  },
  {
    when: function (response) {
      return response.projects.indexOf("hca") > -1;
    },
    type: "input",
    name: "hcaMongoUrl",
    message: "Specify mongodb url for hca app"
  },
  {
    when: function (response) {
      return response.projects.indexOf("eai") > -1;
    },
    type: "input",
    name: "enableOracleIntegration",
    message: "Enable oracle integration (Y/N): "
  },
  {
    when: function (response) {
      return (
        response.projects.indexOf("eai") > -1 &&
        (response.enableOracleIntegration === "Y" ||
          response.enableOracleIntegration === "y")
      );
    },
    type: "input",
    name: "oracleLocation",
    message: "Oracle instant client location"
  },
  {
    when: function (response) {
      return response.projects.indexOf("eai") > -1;
    },
    type: "input",
    name: "enableSapIntegration",
    message: "Enable sap integration (Y/N): "
  },
  {
    when: function (response) {
      return (
        response.projects.indexOf("eai") > -1 &&
        (response.enableSapIntegration === "Y" ||
          response.enableSapIntegration === "y")
      );
    },
    type: "input",
    name: "sapLocation",
    message: "SAP sdk location"
  },
  {
    when: function (response) {
      return response.projects.indexOf("rakeManagement") > -1;
    },
    type: "input",
    name: "rakeManagementMongoUrl",
    message: "Specify mongodb url for rake management app"
  },
  {
    when: function (response) {
      return response.projects.length > 0;
    },
    validate: function (location) {
      try {
        stats = fs.lstatSync(location);

        return stats.isDirectory() ? true : "Location is not a directory";
      } catch (e) {
        return "Location does not exists";
      }
    },
    type: "input",
    name: "installLocation",
    message: "Specify directory location to install applications"
  }
];

function unzipProject(params, name, callback) {
  console.log("extracting project:" + name);
  targz.decompress(
    {
      src: "./" + name + ".tar.gz",
      dest: params.installLocation + "/" + name
    },
    function (err) {
      if (err) {
        console.log(err);
        return;
      } else {
        console.log("project extracted!");
        callback(params, name);
      }
    }
  );
}

function startPm2(params, name) {
  console.log("connecting to pm2 daemon.");
  pm2.connect(function (err) {
    if (err) {
      console.log("connection to pm2 daemon failed: ");
      console.error(err);
      process.exit(2);
    }
    console.log("connected to pm2 daemon.");
    console.log("starting app on pm2");
    pm2.start(params.installLocation + "/" + name + "/ecosystem.config.js",
      function (errStart) {
        pm2.disconnect(); // Disconnect from PM2
        if (errStart) {
          console.log("failed to start app on pm2: ");
          console.log(errStart);
          throw errStart;
        }
        console.log("app started on pm2");
        startPm2Service();
      });
  });
}

function startPm2Service() {
  var pm2ServiceCommand =
    os.platform() === "linux"
      ? 'sudo su -c "pm2 startup linux -u $(whoami) --hp $HOME"'
      : "pm2-startup install";
  console.log("executing :" + pm2ServiceCommand);
  exec(pm2ServiceCommand, function (errorp, stdout, stderr) {
    if (errorp) {
      console.log("pm2-startup error: ");
      console.log(errorp);
      return;
    }
    console.log("output :" + stdout);
    console.log("Error :" + stderr || 'none');
    console.log("pm2-service-install is successful");
    console.log("saving/dumping process list for pm2");
    exec("pm2 save", function (error, out, err) {
      if (error) {
        console.log("processes dumping error");
        console.log(error);
        return;
      }
      console.log("output :" + out);
      console.log("Error :" + err || 'none');
      console.log("processes dumped");
    });
  });
}

function createProcessConfig(params, name) {
  console.log("creating process.json");
  var processConfig = config[name];
  processConfig.env.MONGO_URL = params[name + "MongoUrl"];
  processConfig.env[name.toUpperCase() + "_HOME"] =
    params.installLocation + "/" + name;
  switch (name) {
    case "hcs": {
      processConfig.script =
        params.installLocation + "/" + name + "/src/index.js";
      break;
    }
    case "roadmon": {
      processConfig.script = params.installLocation + "/" + name + "/server.js";
      break;
    }
    default: {
      processConfig.script = params.installLocation + "/" + name + "/main.js";
    }
  }
  processConfig.cwd = params.installLocation + "/" + name;
  var path = process.env.PATH;
  var logFileRoute = envVariable.getHomeDirectory(name);
  processConfig.log_file = logFileRoute.pm2LogFile;
  processConfig.out_file = logFileRoute.pm2OutFile;
  processConfig.error_file = logFileRoute.pm2ErrorFile;

  if (params.oracleLocation && name === "eai") {
    if (os.platform() === "linux") {
      var ldLibraryPath = process.env.LD_LIBRARY_PATH;
      processConfig.env.LD_LIBRARY_PATH =
        params.oracleLocation + ":" + (ldLibraryPath ? ldLibraryPath : "");
    } else {
      path = params.oracleLocation + ";" + (path ? path : "");
    }
  }

  if (params.sapLocation && name === "eai") {
    if (os.platform() === "linux") {
      var sapHome = process.env.SAPNWRFC_HOME;
      processConfig.env.SAPNWRFC_HOME =
        params.sapLocation + ":" + (sapHome ? sapHome : "");
    } else {
      path = params.sapLocation + ";" + (path ? path : "");
    }
  }

  processConfig.env.PATH = path;

  console.log(processConfig);
  try {
    const apps = [processConfig];
    fs.writeFileSync(
      params.installLocation + "/" + name + "/ecosystem.config.js",
      `module.exports = ${JSON.stringify({ apps })}`);
  } catch (e) {
    console.log("Error:" + JSON.stringify(e));
  }
  if (name === "hcs") {
    try {
      // Create HCS dir in GOODSTRACK_HOME and copy config files
      if (!fs.existsSync(`${process.env.GOODSTRACK_HOME}/${name}`)) {
        fs.mkdirSync(`${process.env.GOODSTRACK_HOME}/${name}`);
      }
      if (!fs.existsSync(`${process.env.GOODSTRACK_HOME}/${name}/config`)) {
        fs.mkdirSync(`${process.env.GOODSTRACK_HOME}/${name}/config`);
      }
      fs.copyFileSync(
        `${params.installLocation}/${name}/hcs/config/settings.json`,
        `${process.env.GOODSTRACK_HOME}/${name}/config/settings.json`
      );
    } catch (err) {
      console.error(
        `Error occured in copying HCS settings to GOODSTRACK_HOME \n${err.message}`
      );
    }
  }
  startPm2(params, name);
}

function installProject(params, name) {
  console.log("installing " + name + " project");
  unzipProject(params, name, createProcessConfig);
}

function installNpmbox() {
  targz.decompress(
    {
      src: "./" + os.platform() + "/npmbox.npmbox",
      dest: "./npmbox"
    },
    function (decompressError) {
      if (decompressError) {
        console.log("Error decompressing: " + decompressError);
      } else {
        console.log("extracted npmbox, now installing...");
        var command =
          "npm install --global --cache ./npmbox/.npmbox.cache --optional --cache-min 99999 --shrinkwrap false npmbox";
        console.log("executing: " + command);
        exec(command, function (error, stdout, stderr) {
          if (error) {
            console.log(error);
            console.log("error installing npmbox, exiting...");
            return;
          }
          console.log("stdout: " + stdout);
          console.log("stderr: " + stderr);
          var packageInstallCommand =
            os.platform() === "linux"
              ? "npmunbox " + os.platform() + "/pm2 -g"
              : "npmunbox " +
              os.platform() +
              "/pm2 " +
              os.platform() +
              "/pm2-windows-startup -g";
          console.log("installing via command:" + packageInstallCommand);
          exec(packageInstallCommand, function (errorpm2, stdout2, stderr2) {
            if (errorpm2) {
              console.log(errorpm2);
              console.log("error installing pm2, exiting...");
              return;
            }
            console.log("stdout: " + stdout2);
            console.log("stderr: " + stderr2);
          });
        });
      }
    }
  );
}

function init() {
  inquirer.prompt(questions).then(function (result) {
    result.projects.forEach(function (element) {
      installProject(result, element);
    });
  });
}
init();
